<?php
/**
 * @version		1.0.5
 * @package		Joomla
 * @subpackage	Edocman
 * @author  Tuan Pham Ngoc
 * @copyright	Copyright (C) 2011 Ossolution Team
 * @license		GNU/GPL, see LICENSE.php
 */

// No direct access.
defined('_JEXEC') or die;

jimport('joomla.application.component.modeladmin');

/***** import composant jifile lucene *******/
$path_jifile = JPATH_SITE.'/administrator/components/com_jifile/models/lucene.php';

if(file_exists($path_jifile))
{
	require_once($path_jifile);
}
/********************************************/

/**
 * Edocman model.
 */
class EdocmanModeldocument extends JModelAdmin
{
	/**
	 * @var		string	The prefix to use with controller messages.
	 * @since	1.6
	 */
	protected $text_prefix = 'COM_EDOCMAN';
	
	public function __construct($config = array()) {
	    //Ininitial event for action on document
        if (!isset($config['event_after_delete'])) {
            $config['event_after_delete'] = 'onDocumentAfterDelete' ;
        }	        
	    if (!isset($config['event_after_save'])) {
            $config['event_after_save'] = 'onDocumentAfterSave' ;
        }        
	    if (!isset($config['event_before_delete'])) {
            $config['event_before_delete'] = 'onDocumentBeforeDelete' ;
        }        
	    if (!isset($config['event_before_save'])) {
            $config['event_before_save'] = 'onDocumentBeforeSave' ;
        }
	    if (!isset($config['event_change_state'])) {
            $config['event_change_state'] = 'onDocumentChangeState' ;
        }
        
        parent::__construct($config) ;
	}
	
	/**
	 * Returns a reference to the a Table object, always creating it.
	 *
	 * @param	type	The table type to instantiate
	 * @param	string	A prefix for the table class name. Optional.
	 * @param	array	Configuration array for model. Optional.
	 * @return	JTable	A database object
	 * @since	1.6
	 */
	public function getTable($type = 'Document', $prefix = 'EdocmanTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}

	/**
	 * Method to get the record form.
	 *
	 * @param	array	$data		An optional array of data for the form to interogate.
	 * @param	boolean	$loadData	True if the form is to load its own data (default case), false if not.
	 * @return	JForm	A JForm object on success, false on failure
	 * @since	1.6
	 */
	public function getForm($data = array(), $loadData = true)
	{
		// Initialise variables.
		$app	= JFactory::getApplication();

		// Get the form.
		$form = $this->loadForm('com_edocman.document', 'document', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form)) {
			return false;
		}
		        		
	    if (empty($data))
		    $data = $this->loadFormData() ;
		    		    	
	    if (!$this->canEditState((object) $data)) {
			// Disable fields for display.			
			$form->setFieldAttribute('published', 'disabled', 'true');
            $form->setFieldAttribute('published', 'filter', 'unset');								
		}
								
		$id = JRequest::getInt('id', 0) ;
		if ($id) {
		    $db = & JFactory::getDbo() ;
		    $sql = 'SELECT a.id FROM #__edocman_categories AS a INNER JOIN #__edocman_document_category AS b ON a.id=b.category_id WHERE document_id='.$id ;
		    $db->setQuery($sql) ;		    		    
		    $categoryId = (int) $db->loadResult() ;		    
		}  else {
		    $categoryId = 0 ;		    
		}		

		
		$form->setFieldAttribute('category_id', 'value',  $categoryId);				  
				    		    								
		return $form;
	}

	/**
	 * Method to get the data that should be injected in the form.
	 *
	 * @return	mixed	The data for the form.
	 * @since	1.6
	 */
	protected function loadFormData()
	{
		// Check the session for previously entered form data.
		$data = JFactory::getApplication()->getUserState('com_edocman.edit.document.data', array());

		if (empty($data)) {
			$data = $this->getItem();
		}
        					
		return $data;
	}

	/**
	 * Method to get a single record.
	 *
	 * @param	integer	The id of the primary key.
	 *
	 * @return	mixed	Object on success, false on failure.
	 * @since	1.6
	 */
	public function getItem($pk = null)
	{
		if ($item = parent::getItem($pk)) {
            $item->select_filename = $item->filename ;
		    //Do any procesing on fields here if needed		    
		}

		return $item;
	}

	/**
	 * Prepare and sanitise the table prior to saving.
	 *
	 * @since	1.6
	 */
	protected function prepareTable(&$table)
	{		
		if (empty($table->id)) {

			// Set ordering to the last item if not set
			if (@$table->ordering === '') {
				$db = JFactory::getDbo();								
				$db->setQuery('SELECT MAX(ordering) FROM #__edocman_documents');
				$max = $db->loadResult();
				$table->ordering = $max+1;
			}

		}
	}
		
	/**
	 * Method to save the form data.
	 *
	 * @param	array	The form data.
	 * @return	boolean	True on success.
	 * @since	1.6
	 */
	public function save($data)
	{
	    jimport('joomla.filesystem.file') ;
	    $config = EdocmanHelper::getConfig() ;
		// Initialise variables;
		$app =  JFactory::getApplication() ;
		$dispatcher = JDispatcher::getInstance();
		$table		= $this->getTable();
		$pk			= (!empty($data['id'])) ? $data['id'] : (int)$this->getState($this->getName().'.id');
		$isNew		= true;

		// Include the content plugins for the on save events.
		JPluginHelper::importPlugin('edocman');

		// Load the row if saving an existing category.
		if ($pk > 0) {
			$table->load($pk);
			$isNew = false;
		}
				
		// Alter the title for save as copy
		if (JRequest::getVar('task') == 'save2copy') {
			list($title,$alias) = $this->generateNewTitle($data['parent_id'], $data['alias'], $data['title']);
			$data['title']	= $title;
			$data['alias']	= $alias;
		}
		
		if (JRequest::getVar('task') != 'save2copy') {
    		$categoryId = (int) $data['category_id'] ;
    		if ($categoryId == 0) {
    		    $this->setError(JText::_('EDOCMAN_CHOOSE_CATEGORY')) ;
    		    return false ;
    		}
		}
			    			
		// Bind the data.
		if (!$table->bind($data, array('image', 'filename'))) {
			$this->setError($table->getError());
			return false;
		}					
				
		if (JRequest::getVar('task') != 'save2copy') {
    		//Processing file upload	
    		if ($isNew) {
    		    //We need to check to see whether users uploaded / choosed a file for this document yet
    		    if ((int) $config->file_upload_method == 0) {
        		    if (!$_FILES['jform']['name']['filename'] && empty($data['document_url'])) {
            		    $this->setError(JText::_('EDOCMAN_NO_FILE_UPLOADED'));
            		    return false ;
            		}    
    		    } else {
        		    if (($data['select_filename'] == '' || $data['select_filename'] == '-1') && empty($data['document_url'])) {
            		    $this->setError(JText::_('EDOCMAN_NO_FILE_UPLOADED'));
            		    return false ;
            		}
    		    }
    		}	    		  
		}
		
		//Processing file upload				
	    if(is_uploaded_file($_FILES['jform']['tmp_name']['filename'])) 
		{	
			$db = $this->getDbo();
			$sql = 'SELECT `path` FROM #__edocman_categories WHERE id = '.$categoryId ;
			$db->setQuery($sql);
			$path = $db->loadResult();
			
			$rootDir =  $config->documents_path;
			//dump($rootDir, "rootdir");
			$fileName = JFile::makeSafe($_FILES['jform']['name']['filename']);
			
		    /*** vrification nom du fichier ***/
		    if($config->require_ascii_filename)
		    {
		    	//compte le nombre de points dans la chane
		    	$nb_point = substr_count($fileName, '.');
		    	
		    	if($nb_point == 1)
		    	{
		    		$fileNameSansExt = JFile::stripExt($fileName);
		    		 
		    		if(JRequest::getVar('task') != 'save2copy')
		    		{
		    			if(preg_match("#[^a-zA-Z0-9_-]#", $fileNameSansExt))
		    			{
		    				$this->setError(JText::_('EDOCMAN_NO_ASCII_FILENAME')) ;
		    				return false;
		    			}
		    		}
		    	}
		    	
		    	else
		    	{
		    		$this->setError(JText::_('EDOCMAN_NO_ASCII_FILENAME')) ;
		    		return false;
		    	}
		    }
		    /***************************/
		    
		    if ($path) 
		    {
		        $savedFilePath = $rootDir.'/'.$path.'/'.$fileName;
		    } 
		    
		    else                                      
		    {
		        $savedFilePath = $rootDir.'/'.$fileName;
		    }	
		    	    
		    if (JFile::exists($savedFilePath))
		    {
		        $savedFilename = uniqid('file_').'_'.$fileName ;
		    }

		    else 
		    {
		        $savedFilename = $fileName;  
		    }
			
		    $categoryId = (int)$data['category_id'];
			
		    if ($path) {
		        JFile::upload($_FILES['jform']['tmp_name']['filename'], $rootDir.'/'.$path.'/'.$savedFilename) ;
		    } else {
		        JFile::upload($_FILES['jform']['tmp_name']['filename'], $rootDir.'/'.$savedFilename) ;    
		    }            
		    $table->original_filename = $_FILES['jform']['name']['filename'];
		    if ($path)
		        $table->filename = $path.'/'.$savedFilename ;		    		        
		    else 
		        $table->filename = $savedFilename ; 
		} elseif (isset($data['select_filename']) && $data['select_filename'] && ($data['select_filename'] != '-1')) {
		    $table->original_filename = $table->filename = $data['select_filename'] ;		    
		}
						
		if (is_uploaded_file($_FILES['jform']['tmp_name']['image'])) {		     
		     $canUpload = true ;
		     if (!getimagesize($_FILES['jform']['tmp_name']['image'])) {
		         $app->enqueueMessage(JText::_('COM_EDOCMAN_IMAGE_ERROR'), 'error') ;
		         $canUpload = false ;		         
		     }		     
		     //XSS checking
    		$xss_check =  JFile::read($_FILES['jform']['tmp_name']['image'], false, 256);
    		$html_tags = array('abbr', 'acronym', 'address', 'applet', 'area', 'audioscope', 'base', 'basefont', 'bdo', 'bgsound', 'big', 'blackface', 'blink', 'blockquote', 'body', 'bq', 'br', 'button', 'caption', 'center', 'cite', 'code', 'col', 'colgroup', 'comment', 'custom', 'dd', 'del', 'dfn', 'dir', 'div', 'dl', 'dt', 'em', 'embed', 'fieldset', 'fn', 'font', 'form', 'frame', 'frameset', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'head', 'hr', 'html', 'iframe', 'ilayer', 'img', 'input', 'ins', 'isindex', 'keygen', 'kbd', 'label', 'layer', 'legend', 'li', 'limittext', 'link', 'listing', 'map', 'marquee', 'menu', 'meta', 'multicol', 'nobr', 'noembed', 'noframes', 'noscript', 'nosmartquotes', 'object', 'ol', 'optgroup', 'option', 'param', 'plaintext', 'pre', 'rt', 'ruby', 's', 'samp', 'script', 'select', 'server', 'shadow', 'sidebar', 'small', 'spacer', 'span', 'strike', 'strong', 'style', 'sub', 'sup', 'table', 'tbody', 'td', 'textarea', 'tfoot', 'th', 'thead', 'title', 'tr', 'tt', 'ul', 'var', 'wbr', 'xml', 'xmp', '!DOCTYPE', '!--');
    		foreach($html_tags as $tag) {
    			// A tag is '<tagname ', so we need to add < and a space or '<tagname>'
    			if(stristr($xss_check, '<'.$tag.' ') || stristr($xss_check, '<'.$tag.'>')) {
    				JError::raiseWarning(100, JText::_('IE XSS Warning'));
    				$canUpload = false ;    				
    			}    			
    		}    
    		$ret = true ;		     		
    		if ($canUpload) {
    		    //Process upload if there are no error found
    		    $documentImagePath = JPATH_ROOT.'/media/com_edocman/document/';
    		    $thumbDocumentImagePath = $documentImagePath.'thumbs/' ;
    		    $fileName = uniqid('category_').$_FILES['jform']['name']['image'];    		    
    		    $ret = JFile::upload($_FILES['jform']['tmp_name']['image'], $documentImagePath.$fileName) ;
    		    if ($ret) {
    		        $width = $config->document_thumb_width ? $config->document_thumb_width : 100 ;
    		        $height = $config->document_thumb_height ? $config->document_thumb_height : 100 ;    		        
    		        $ret = EdocmanHelper::resizeImage($documentImagePath.$fileName, $thumbDocumentImagePath.$fileName, $width, $height, 95) ;    		        
    		    }        		    
    		}	     	
    		if ($ret) 
			{    		 
    		    if($table->image) 
    		    {
    		        if (JFILE::exists($documentImagePath.$table->image))    
    		            JFile::delete($documentImagePath.$table->image) ;
    		        if (JFILE::exists($thumbDocumentImagePath.$table->image))    
    		            JFile::delete($thumbDocumentImagePath.$table->image) ;        		            		            		       
    		    }       		    
    		    $table->image = $fileName ;    		    
    		}	        		    
		}
		
		/****** fonction ajout enregistrer le contenu du document dans la description*****/
		//si le composant existe
		if(file_exists(JPATH_SITE.'/administrator/components/com_jifile/models/lucene.php'))
		{
			$component = JComponentHelper::getComponent("com_jifile", $strict);
			
			//si le composant est activer
			if($component->enabled)
			{
				$rootDir =  $config->documents_path;

				// si le filename(category/nomfichier.extension) enregistrer est rempli
				if($table->filename != null)
				{
					//dump($rootDir."/".$table->filename, "Root + filename");
					
					$fields = array('title'=>'', 'subject'=>'', 'description'=>'', 'creator'=>'', 'keywords'=>'', 'body'=>'');
					
					$model = new JifileModelLucene();
					
					//recupration du document via le path($savedFilePath)
					$ifiledoc = $model::getLuceneDoc($rootDir."/".$table->filename, true, false);
					$ifiledoc = array_merge($fields, (empty($ifiledoc) ? array() : $ifiledoc));
					
					$data['description'] = htmlentities($ifiledoc['body'], ENT_COMPAT);
					//ajout de la donne description dans la table a sauvegarder
					$table->description = $data['description'];
				}
			}
		}
		/******************************************************************************/
						
		// Check the data.
		if (!$table->check()) {
			$this->setError($table->getError());
			return false;
		}

		// Trigger the onContentBeforeSave event.
		$result = $dispatcher->trigger($this->event_before_save, array($this->option.'.'.$this->name, &$table, $isNew));
		if (in_array(false, $result, true)) {
			$this->setError($table->getError());
			return false;
		}

		// Store the data.
		if (!$table->store()) {
			$this->setError($table->getError());
			return false;
		}
		
		$dispatcher->trigger($this->event_after_save, array($this->option.'.'.$this->name, &$table, $isNew));
			
		$this->setState($this->getName().'.id', $table->id);

		// Clear the cache
		$this->cleanCache();

		return true;
	}	

	
	protected function canDelete($record)
	{		
		if ($record->asset_id == 0) {
			return true ;
		} 	
		if (!empty($record->id)) {			
			$user = JFactory::getUser();			
			return $user->authorise('core.delete', 'com_edocman.document.'.(int) $record->id);
		}
	}
	
	
	protected function canEditState($record)
	{
		$user = JFactory::getUser();

		// Check for existing article.
		if (!empty($record->id)) {
			return $user->authorise('core.edit.state', 'com_edocman.document.'.(int) $record->id);
		}
		// New article, so check against the category.
		elseif (!empty($record->category_id)) {
			return $user->authorise('core.edit.state', 'com_edocman.category.'.(int) $record->catid);
		}
		// Default to component settings if neither article nor category known.
		else {
			return parent::canEditState($record);
		}
	}
	
	
}